from json import loads
from collections import OrderedDict
from re import match
from functools import reduce

prog = r'''
{
	"main" : {
		"val" : ["expect val", "inp"],
		"main":
			["and",
				["[]", "val", 0],
				["not", ["trim left", ["[]", "val", 1]]]
			]
	},

	"valid" : ["def", ["s"], ["list", true, "s"]],

	"invalid" : ["lit", [false, ""]],

	"expect val" : ["def", ["s"], {
		"trimmed" : ["trim left", "s"],
		"main" : ["if", "trimmed",
			["match", ["[]", "trimmed", 0],
				[["{"], ["expect obj", "trimmed"]],
				[["["], ["expect arr", "trimmed"]],
				[["\""], ["expect str", "trimmed"]],
				[["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "-"],
					["expect by regex",
						"trimmed",
						["lit", "-?(?:0|[1-9][0-9]*)(?:\\.[0-9]+)?(?:[eE][+-]?[0-9]+)?"]
					]
				],
				[["t"], ["expect by regex", "trimmed", ["lit", "true"]]],
				[["f"], ["expect by regex", "trimmed", ["lit", "false"]]],
				[["n"], ["expect by regex", "trimmed", ["lit", "null"]]],

				[false, "invalid"]
			],
			"invalid"
		]
	}],

	"expect by regex" : ["def", ["s", "regex"], {
		"matched" : ["regex match", "regex", "s"],
		"main" : ["match", "matched",
			[[null], "invalid"],
			[false, ["valid", ["[a:]", "s", ["len", "matched"]]]]
		]
	}],

	"expect list of" : ["def", ["s", "expect item"], {
		"matched first" : ["expect item", "s"],
		"res" : null,
		"main" : ["if", ["[]", "matched first", 0],
			["do",
				["set", "s", ["[]", "matched first", 1]],
				["loop", ["not", "res"], {
					"matched comma" : ["expect by regex", ["trim left", "s"], ["lit", ","]],
					"matched item" : ["expect item", ["[]", "matched comma", 1]],

					"did match comma" : ["[]", "matched comma", 0],
					"did match item" : ["[]", "matched item", 0],

					"main" : ["match", ["list", "did match comma", "did match item"],
						[[[true, true]], ["set", "s", ["[]", "matched item", 1]]],
						[[[false, false]], ["set", "res", ["valid", "s"]]],
						[false, ["set", "res", "invalid"]]
					]
				}],
				"res"
			],
			["valid", "s"]
		]
	}],

	"expect obj" : ["def", "s", {
		"matched contents" : ["expect list of", ["[a:]", "s", 1], "expect pair"],
		"s" : ["trim left", ["[]", "matched contents", 1]],
		"main" : ["if", "s",
			["match", ["[]", "s", 0],
				[["}"], ["valid", ["[a:]", "s", 1]]],
				[false, "invalid"]
			],
			"invalid"
		]
	}],

	"expect pair" : ["def", ["s"], {
		"matched key" : ["expect str", ["trim left", "s"]],
		"matched colon" : ["expect by regex",
			["trim left", ["[]", "matched key", 1]],
			["lit", ":"]
		],
		"matched val" : ["expect val", ["[]", "matched colon", 1]],

		"did match key" : ["[]", "matched key", 0],
		"did match colon" : ["[]", "matched colon", 0],
		"did match val" : ["[]", "matched val", 0],

		"main" : ["if",
			["and", "did match key", "did match colon", "did match val"],
			"matched val",
			"invalid"
		]
	}],

	"expect arr" : ["def", "s", {
		"matched contents" : ["expect list of", ["[a:]", "s", 1], "expect val"],
		"s" : ["trim left", ["[]", "matched contents", 1]],
		"main" : ["if", "s",
			["match", ["[]", "s", 0],
				[["]"], ["valid", ["[a:]", "s", 1]]],
				[false, "invalid"]
			],
			"invalid"
		]
	}],

	"expect str" : ["def", "s",
		["expect by regex",
			"s",
			["lit", "\"(?:[^\"\\\\\\n]|\\\\[\"\\\\/bfnrt]|\\\\u[0-9a-fA-F]{4})*\""]
		]
	],

	"trim left" : ["def", ["s"],
		["[]", ["expect by regex", "s", ["lit", "\\s*"]], 1]
	]
}
'''

parsed = loads(prog, object_pairs_hook=OrderedDict)

## ¿Por qué no ha sido aceptado el PEP 505?
def regex_match(r, s):
	m = match(r, s)
	if m: return m.group()

fs = {
	'+' : lambda f, s: f + s,
	'-' : lambda f, s: f - s,
	'*' : lambda f, s: f * s,
	'/' : lambda f, s: f / s,
	'=' : lambda f, s: f == s,
	'!=' : lambda f, s: f != s,

	'and': lambda *aas: reduce(lambda f, s: f and s, aas, True),
	'or': lambda *aas: reduce(lambda f, s: f or s, aas, False),
	'not': lambda f: not f,

	'[]' : lambda v, i: v[i],
	'[:]' : lambda v, si, ei: v[si:ei],
	'[a:]' : lambda v, si: v[si:],
	'[:b]' : lambda v, ei: v[:ei],

	'list' : lambda *aas: list(aas),
	'len' : lambda v: len(v),
	'in' : lambda v, l: v in l,

	'regex match': regex_match,
}

def entry(j):
	return run(parsed, [{'inp' : j}, fs])

def run(p, ss):
	## Diccionario es como let ... in ...
	if type(p) == OrderedDict:
		ss = [{}] + ss
		for k, v in p.items():
			if k != 'main':
				ss[0][k] = run(v, ss)
		return run(p['main'], ss)
	## Lista llama a una función o es una forma especial, que puede ser:
	## - Un valor literal (ya que una cadena normal hace referencia a una variable, una lista tiene sintaxis especial, etc.)
	## - Una función anónima
	## - Un condicional
	## - Una expresión 'match', que empareja un valor con el que corresponde
	## - Un bucle
	## - Un bloque
	## - Cambiar el valor de una variable
	elif type(p) == list:
		fn = p[0]
		if fn == 'lit':
			return p[1]
		elif fn == 'def':
			return lambda *aas: run(
				p[2],
				[{a : aas[i] for i, a in enumerate(p[1])}] + ss
			)
		elif fn == 'if':
			if run(p[1], ss):
				return run(p[2], ss)
			else:
				return run(p[3], ss)
		elif fn == 'match':
			v = run(p[1], ss)
			for c, r in p[2:]:
				if not c or v in c:
					return run(r, ss)
			raise 'Expresión \'match\' no encontró un valor correcto'
		elif fn == 'loop':
			v = None
			while run(p[1], ss):
				v = run(p[2], ss)
			return v
		elif fn == 'do':
			for e in p[1:-1]:
				run(e, ss)
			return run(p[-1], ss)
		elif fn == 'set':
			v = run(p[2], ss)
			set(p[1], v, ss)
			return v
		else:
			if False:
				a = list(map(lambda e: run(e, ss), p[1:]))
				print([fn] + a, p)
				return run(fn, ss)(*a)
			else:
				return run(fn, ss)(*map(lambda e: run(e, ss), p[1:]))
	## Cadena de caracteres devuelve el valor de una variable
	elif type(p) == str:
		return get(p, ss)
	## Números, booleanos y nulo evaluan a ellos mismos
	elif type(p) == int or type(p) == float or type(p) == bool or p is None:
		return p
	else:
		raise '¿Cómo?'

## Los 'scopes' van de más interior a más exterior
def get(v, ss):
	for s in ss:
		if v in s:
			## print(v, s[v])
			return s[v]

def set(v, vv, ss):
	for s in ss:
		if v in s:
			s[v] = vv
			## print(v, vv, ss)

if __name__ == '__main__':
	from sys import argv
	if len(argv) > 1:
		print(entry(open(argv[1]).read()))
	else:
		while True:
			print(entry(input()))